local LDB

local menu = {}
local parent = CreateFrame('Frame')
local startChar = {
	["A-D"] = {
	},
	["E-H"] = {
	},
	["I-L"] = {
	},
	["M-P"] = {
	},
	["Q-T"] = {
	},
	["U-X"] = {
	},
	["YZ"] = {
	}
}

local function pairsByKeys (startChar, f)
	local a = {}
		for n in pairs(startChar) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], startChar[a[i]]
			end
		end
	return iter
end

local function UpdateDisplay()
	if lastmountIndex ~= nil then
		LDB.text = lastmountText
		LDB.icon = lastmountIcon
	end
	
	for i = 1, GetNumCompanions("MOUNT") do
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i)
		if issummoned then
			LDB.text = creatureName
			LDB.icon = icon
			lastmountIcon = icon
			lastmountIndex = i
			lastmountSpell = creatureSpellID
			lastmountText = creatureName
			return
		end
		if lastmountSpell ~= nil then
		local isUsable, notEnoughMana = IsUsableSpell(lastmountSpell)
			if not isUsable then
				LDB.text = string.format('|cff848484%s|r', lastmountText)
			else
				LDB.text = lastmountText
			end
		end
	end
end

local function ModifiedClick(button, i)	
	if IsShiftKeyDown() then
	PickupCompanion("MOUNT", i)
	else
	CallCompanion("MOUNT", i)
	end
end

local function OnTooltipShow(self)
	self:AddLine('|cff00ffffBrokerMounts|r')
	self:AddLine("<Right Click> to open mount list")
	self:AddLine("<Left Click> to mount/dismount")
	self:AddLine("<Shift-Left Click> on mount to pick it up")
	if GetNumCompanions("MOUNT") == 0 then
	self:AddLine("|cffff0000You have no mounts|r")
	else
	self:AddLine(string.format("|cff00ff00You have %s mounts|r", GetNumCompanions("MOUNT")))
	end
end

local function OnClick(self, button)
	if button == 'RightButton' then
		ToggleDropDownMenu(1, nil, parent, self, 0, 0)
	end
	if button == 'LeftButton' then
		if IsMounted() then
			Dismount()
		elseif lastmountIndex ~= nil then
			CallCompanion("MOUNT", lastmountIndex)
		end
	end
end

local function CreateMenu(self, level)
	menu = wipe(menu)
	local firstChar
	if 	GetNumCompanions("MOUNT") <= 30 then
	for i = 1, GetNumCompanions("MOUNT") do
		local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i)
		local isUsable, notEnoughMana = IsUsableSpell(creatureSpellID)
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		menu.notCheckable = 1
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		UIDropDownMenu_AddButton(menu)
	end
	elseif GetNumCompanions("MOUNT") >= 31 then
	level = level or 1;
	if (level == 1) then
	for key, v in pairsByKeys(startChar) do
		menu.text = key
		menu.notCheckable = 1
		menu.hasArrow = true;
		menu.value = {
			["Level1_Key"] = key;
		};
	
		UIDropDownMenu_AddButton(menu, level)
	end
	end
	if (level == 2) then
	 -- getting values of first menu
    local Level1_Key = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"];
		
    for i = 1, GetNumCompanions("MOUNT") do
		
		local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i)
		local isUsable, notEnoughMana = IsUsableSpell(creatureSpellID)
		firstChar = strupper(strsub(creatureName, 1, 1))

		if firstChar >= "A" and firstChar <= "D" and Level1_Key == "A-D" then
		menu.hasArrow = false;
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
		
		if firstChar >= "E" and firstChar <= "H" and Level1_Key == "E-H" then
		menu.hasArrow = false; 
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
		
		if firstChar >= "I" and firstChar <= "L" and Level1_Key == "I-L" then
		menu.hasArrow = false;
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
		
		if firstChar >= "M" and firstChar <= "P" and Level1_Key == "M-P" then
		menu.hasArrow = false;
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
		
		if firstChar >= "Q" and firstChar <= "T" and Level1_Key == "Q-T" then
		menu.hasArrow = false;
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
	
		if firstChar >= "U" and firstChar <= "X" and Level1_Key == "U-X" then
		menu.hasArrow = false;
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
		
		if firstChar >= "Y" and firstChar <= "Z" and Level1_Key == "YZ" then
		menu.hasArrow = false; 
		menu.notCheckable = true;
		menu.text = string.format('|T%s:26|t %s', icon, creatureName)
		menu.func = ModifiedClick
		menu.arg1 = i
		
		if not isUsable then
			menu.text = string.format('|T%s:26|t |cff848484%s|r', icon, creatureName)
		end
		if issummoned then
			menu.text = string.format('|T%s:26|t |cff00ff00%s|r', icon, creatureName)
		end
		
		UIDropDownMenu_AddButton(menu, level)
		end
	
	end
	end
	end

end

function parent:PLAYER_LOGIN()
	LDB = LibStub('LibDataBroker-1.1'):NewDataObject('BrokerMounts', {
		icon = "Interface\\Icons\\INV_Misc_QuestionMark.blp",
		text = "BrokerMounts",
		type = 'data source',
		OnTooltipShow = OnTooltipShow,
		OnClick = OnClick,
	})
	self:RegisterEvent('SPELL_UPDATE_USABLE')
	self:RegisterEvent('COMPANION_UPDATE')
	self.COMPANION_UPDATE = UpdateDisplay
	self.SPELL_UPDATE_USABLE = UpdateDisplay

	self.initialize = CreateMenu
	self.displayMode = 'MENU'
	
	UpdateDisplay()
end

parent:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
parent:RegisterEvent('PLAYER_LOGIN')